<?php

namespace Drupal\noahs_page_builder;

use Drupal\noahs_page_builder\Controls_Base;
use Drupal\file\Entity\File;
use Drupal\image\Entity\ImageStyle;
use Drupal\Core\Render\Markup;
use Drupal\node\Entity\Node;
use Drupal\media\Entity\Media;


class Controls_Manager {

	private $register;

	/**
	 * Text control.
	 */
	const TEXT = 'text';

	/**
	 * Text control.
	 */
	const INFO = 'info';

	/**
	 * Color control.
	 */
	const NOAHS_COLOR = 'noahs_color';

	/**
	 * Background control.
	 */
	const NOAHS_VIDEO_UPLOAD = 'noahs_video_upload';

	/**
	 * Background control.
	 */
	const NOAHS_BACKGROUND_IMAGE = 'noahs_background_image';

	/**
	 * Background control.
	 */
	const NOAHS_BACKGROUND_GRADIENT = 'noahs_background_gradient';
	/**
	 * Background control.
	 */
	const NOAHS_BACKGROUND_OVERLAY = 'noahs_background_overlay';
	/**
	 * Background control.
	 */
	const NOAHS_VIDEO_BACKGROUND = 'noahs_video_background';
	/**
	 * Background control.
	 */
	const NOAHS_IMAGE = 'noahs_image';
	/**
	 * MArgin control.
	 */
	const NOAHS_MARGIN = 'noahs_margin';
	/**
	 * Padding control.
	 */
	const NOAHS_PADDING = 'noahs_padding';
	/**
	 * Padding control.
	 */
	const NOAHS_BORDER_RADIUS = 'noahs_radius';
	/**
	 * Padding control.
	 */
	const NOAHS_BORDER = 'noahs_border';
	/**
	 * Padding control.
	 */
	const NOAHS_GROUP_RADIO = 'noahs_group_radio';
	/**
	 * Padding control.
	 */
	const NOAHS_GROUP_CHECKBOX = 'noahs_group_checkbox';
	/**
	 * Padding control.
	 */
	const NOAHS_WIDTH = 'noahs_width';
	/**
	 * Padding control.
	 */
	const NOAHS_MULTIPLE_ELEMENTS = 'noahs_multiple_elements';
	/**
	 * Padding control.
	 */
	const NOAHS_COORDINATES = 'noahs_coordinates';
	/**
	 * Padding control.
	 */
	const NOAHS_TRANSFORM = 'noahs_transform';
	/**
	 * Padding control.
	 */
	const NOAHS_GALLERY = 'noahs_gallery';
	/**
	 * Padding control.
	 */
	const NOAHS_CAROUSEL = 'noahs_carousel';
	/**
	 * Padding control.
	 */
	const NOAHS_ICON = 'noahs_icon';
	/**
	 * Padding control.
	 */
	const NOAHS_IMAGE_MASK = 'noahs_image_mask';
	/**
	 * Padding control.
	 */
	const NOAHS_SELECT_DESIGN = 'noahs_select_design';
	/**
	 * Padding control.
	 */
	const HTML = 'html';
	/**
	 * Number control.
	 */
	const NUMBER = 'number';

	/**
	 * Textarea control.
	 */
	const TEXTAREA = 'textarea';

	/**
	 * Select control.
	 */
	const SELECT = 'select';

	/**
	 * Select control.
	 */
	const CHECKBOX = 'checkbox';

	/**
	 * Select control.
	 */
	const RADIO = 'radio';

	/**
	 * Hidden control.
	 */
	const HIDDEN = 'hidden';

	/**
	 * Tab control.
	 */
	const TAB = 'tab';

	/**
	 * Tabs control.
	 */
	const TABS = 'tabs';

	/**
	 * Divider control.
	 */
	const NOAHS_DIVIDER = 'noahs_divider';

	/**
	 * Divider control.
	 */
	const GROUP = 'group';

	/**
	 * Font control.
	 */
	const NOAHS_FONT = 'noahs_font';


	const DATE = 'date';
	const NOAHS_SHADOWS = 'noahs_shadows';
	const NOAHS_CUSTOM_CSS = 'noahs_custom_css';
	const NOHAS_URL = 'noahs_url';




	/**
	 * Controls.
	 *
	 * Holds the list of all the controls. Default is `null`.
	 *
	 * @since 1.0.0
	 * @access private
	 *
	 * @var Base_Control[]
	 */
	private $controls = null;





	public static function get_controls_names() {
		return [
			self::TABS,
			self::TAB,
			self::GROUP,
			self::TEXT,
			self::DATE,
			self::NUMBER,
			self::HTML,
			self::SELECT,
			self::CHECKBOX,
			self::INFO,
			self::RADIO,
			self::HIDDEN,
			self::TEXTAREA,
			self::NOAHS_BACKGROUND_IMAGE,
			self::NOAHS_BACKGROUND_GRADIENT,
			self::NOAHS_BACKGROUND_OVERLAY,
			self::NOAHS_IMAGE,
			self::NOAHS_VIDEO_BACKGROUND,
			self::NOAHS_MARGIN,
			self::NOAHS_PADDING,
			self::NOAHS_SHADOWS,
			self::NOAHS_BORDER_RADIUS,
			self::NOAHS_GROUP_RADIO,
			self::NOAHS_GROUP_CHECKBOX,
			self::NOAHS_COLOR,
			self::NOAHS_FONT,
			self::NOAHS_WIDTH,
			self::NOAHS_BORDER,
			self::NOAHS_MULTIPLE_ELEMENTS,
			self::NOAHS_COORDINATES,
			self::NOAHS_GALLERY,
			self::NOAHS_CAROUSEL,
			self::NOAHS_ICON,
			self::NOAHS_IMAGE_MASK,
			self::NOAHS_DIVIDER,
			self::NOAHS_CUSTOM_CSS,
			self::NOHAS_URL,
			self::NOAHS_SELECT_DESIGN,
			self::NOAHS_TRANSFORM,
			self::NOAHS_VIDEO_UPLOAD,
		];
	}

	/**
	 * Register controls.
	 */

	private function register_controls() {
		$this->controls = [];
		foreach ( self::get_controls_names() as $control_id ) {
			$control_class_id = str_replace( ' ', '_', ucwords( str_replace( '_', ' ', $control_id ) ) );
	
		}
	}

	public function register( $control_instance, $control_id = null ) {

		$control_id = $control_instance->get_type();

		$this->controls[ $control_id ] = $control_instance;
	}

	/**
	 * Get controls.
	 */
	public function get_controls() {

		if ( null === $this->controls ) {
			$this->register_controls();
		}

		return $this->controls;
	}

	/**
	 * Get control.
	 */

	public function get_control( $control_id ) {

		$controls = $this->get_controls();

		return isset( $controls[ $control_id ] ) ? $controls[ $control_id ] : false;
	}


	/**
	 * Render tabs with inputs iside.
	 */
	public function render_tabs($tabs, $values, $multiple = null, $delta = null, $parent = null) {

		
		$html_tabs = '';
		$css = '';

		$control_tabs = new Control_Tabs();
		$default_fields = new Controls_Base;
	 	$tab_titles = [];
		$items = [];


		if(!$multiple){
			$array1 = $default_fields->defaultFields();
			$array1 = $default_fields->groupFields($array1);

			foreach ($array1 as $key => $value) {


				if (array_key_exists($key, $tabs)) {
				
					$tabs[$key]['items'] = array_merge(
						$tabs[$key]['items'],
						array_diff_key($value['items'], $tabs[$key]['items'])
					);
				
				} else {
			
					$tabs[$key] = $value;
				}
			}
		}


		$first = true;

		foreach($tabs as $tab_id => $tab){
			$html_tab = '';
	
			foreach($tab['items'] as $item_id => $item){
				
				if(isset($item['type']) && $item['type'] === 'group'){
					$group = new Control_Group();

					$html_group = '';
					foreach ($item['items'] as $group_item_id => $group_item) {


						$group_items = [
							'item' => $group_item,
							'wid' => $values ? $values['wid'] : null,
							'item_id' => $group_item_id,
							'delta' => $delta,
							'parent' => $parent,
							'values' => $values,
						];
						$html_group .= $this->render_controls($group_items, $values['element'], 'group', $delta);
					}
					$html_tab  .= $group->content_template($html_group, $item_id, $item['title']);
				}else{
					$items[$item_id] = $item;

					$data = [
						'item' => $item,
						'wid' => isset($values['wid']) ? $values['wid'] : null,
						'item_id' => $item_id,
						'delta' => $delta,
						'parent' => $parent,
						'values' => $values,
					];

					$values_element = isset($values['element']) ? $values['element'] : [];

					$html_tab .= $this->render_controls($data, $values_element, '', $delta);
				}
			}

			$tab_titles[$tab_id] = $tab['title'];
			$control_tab = new Control_Tab();

			$html_tabs .= $control_tab->content_template($html_tab,$tab_id, $tab['title'], $delta, $first); // Agregar el HTML del tab al HTML de todos los tabs
			$first = false;
		}

		return ['form' => $control_tabs->content_template($html_tabs, $tab_titles, $delta)];
	}

	public function render_controls($data, $values, $wrapper = null, $delta = null) {
		$html_form_fields = new Controls_Base;

		return $html_form_fields->extractHtml($data, $values, $wrapper, $delta);
	}


	// Get Classes from selector
	public function getClasses($items, $values, $wid) {

		foreach($values as $item_id => $element){
			if(!empty($element)){
				if(!empty($items[$item_id]['style_selector']) && $items[$item_id]['style_selector'] === 'widget'){
					$class['#widget-id-'.$wid ][] = $element;
				}else if(!empty($items[$item_id]['style_selector'])){
					$class['#widget-id-'.$wid .' '. $items[$item_id]['style_selector']][] =  $element;
				}
			}
		}

		if(!empty($class)){
			return $this->transformArray($class);
		}
	}

	// Get Attributes from selector
	public function getAttributes($items, $values, $wid) {
		$class = [];

		foreach($values as $item_id => $element){
		
			if(isset($element)){
				if($items[$item_id]['style_type'] === 'attribute'){
					if(!empty($element['text'])){
						$element =  $element['text'];
					}
				
					if($items[$item_id]['style_selector'] === 'widget'){
						$class['#widget-id-'.$wid ][$items[$item_id]['attribute_type']] = $element;
					}else{
						$class[ '#widget-id-'.$wid .' '. $items[$item_id]['style_selector']][$items[$item_id]['attribute_type']] =  $element;
					}
				
				}
			}
		}
	
		return $class;

	}

	// Transform Array to implode
	public function transformArray($array) {
		$newArray = [];

		foreach ($array as $key => $value) {
			if (is_array($value)) {
				$flattenedValues = [];
	
				foreach ($value as $val) {
					if (is_array($val)) {
						$flattenedValues[] = implode(' ', $val);
					} else {
						$flattenedValues[] = $val;
					}
				}
	
				$newArray[$key] = implode(' ', $flattenedValues);
			} else {
				$newArray[$key] = $value;
			}
		}
	
		return $newArray;
	}

	// getStyles Principal Function
	public function getStyles($items, $value, string $wid) {


		$style = [];
		$css = "";

		$querys = Controls_Base::getMediaQuery();
		
		if(!empty($value)){
		
			foreach($value as $query => $item_value){
		
				if($query === 'desktop'){
					foreach($item_value as $state => $state_value){		
							
						if($state === 'default'){
		
							$css .= $this->generateCSS($state, $state_value, $items, $wid);

						}else{
							$css .= $this->generateCSS($state, $state_value, $items, $wid);
						}
					}
				}
				if($query === 'tablet'){
					$css .= '@media (max-width:959px){';
					foreach($item_value as $state => $state_value){						
						if($state === 'default'){
							$css .= $this->generateCSS($state, $state_value, $items, $wid);
						}else{
							$css .= $this->generateCSS($state, $state_value, $items, $wid);
						}
					}
					$css .= '}';
				}
				if($query === 'mobile'){
					$css .= '@media (max-width:767px){';
					foreach($item_value as $state => $state_value){						
						if($state === 'default'){
							$css .= $this->generateCSS($state, $state_value, $items, $wid);
						}else{
							$css .= $this->generateCSS($state, $state_value, $items, $wid);
						}
					}
					$css .= '}';
				}

			}
		}
		return $css;
		
	}

	public function getBackgroundImage($value){
		$css = "";
		$tokenService = \Drupal::token();

		
		if(!empty($value['token_media'])){
			$token_value = $value['token_media'];
		}

		if(!empty($value['token_media'])){

			$route_match = \Drupal::routeMatch();
			if(!empty($route_match->getParameter('entity')) && !empty($route_match->getParameter('nid'))){
				$node = \Drupal::entityTypeManager()->getStorage($route_match->getParameter('entity'))->load($route_match->getParameter('nid'));
				$rendered_token = '';
				if(!empty($node)){
				$rendered_token = $tokenService->replace($token_value, [
						$route_match->getParameter('entity') => $node,
					]);
				}else{
				$rendered_token = $tokenService->replace($token_value);
				}
			}else if(!empty($route_match->getParameter('node'))){
				$node = $route_match->getParameter('node');
				$rendered_token = '';
				if(!empty($node)){
				$rendered_token = $tokenService->replace($token_value, [
					$node->getEntityTypeId() => $node,
					]);
				}else{
				$rendered_token = $tokenService->replace($token_value);
				}
			}
			

			$css .= 'background-image:' .'url(' . $rendered_token . ');';
		

		}
	 
		
		if(!empty($value['background_image']['fid'])){
			$media = Media::load($value['background_image']['fid']);
            if ($media && $media->bundle() === 'image') {
               $media_field_name = 'field_media_image'; 
               if ($media->hasField($media_field_name) && !$media->get($media_field_name)->isEmpty()) {
                  $file = $media->get($media_field_name)->entity;
					if (!empty($file) && $file != NULL && !is_null($file->getFileUri())) {
						$file_uri = $file->getFileUri();
						if(!empty($value['image_style'])){
							$background_image = ImageStyle::load($value['image_style'])->buildUrl($file_uri);
						}else{
							$background_image =  \Drupal::service('file_url_generator')->generateAbsoluteString($file_uri);
						}
						$css .= 'background-image:' .'url(' . $background_image . ');';
					}
				}
			}
		}

		$css .= 'background-repeat:' . ($value['background_image']['background_repeat'] ?? 'no-repeat') . ';';
		$css .= 'background-position:' . ($value['background_image']['background_position'] ?? 'center center'). ';';
		$css .= 'background-attachment:' . ($value['background_image']['background_attachment'] ?? 'initial'). ';';

		if(!empty($value['background_size_custom'])){
			$css .= 'background-size:' .  $value['background_size_custom'] . ';';
		}else{
			$css .= 'background-size:' .  ($value['background_image']['background_size'] ?? 'cover'). ';';
		}


		return $css;
	}


	public function dividerStyles($k, $divider) {
		$css = "";
		$config = \Drupal::config('noahs_page_builder.settings');
		if (!empty($divider['divider'])) {
			$css .= 'width:' . (!empty($divider['width']) ? $divider['width'] : '100') . '%;';
			$css .= 'height:' . (!empty($divider['height']) ? $divider['height'] : '300') . 'px;';
			
			$css .= 'fill:' . ($divider['color'] ?? $config->get('principal_color')) . ';';
			
			if(!empty($divider['align'])){
				if($divider['align'] === 'center'){
					$css .= 'margin:auto;';
				}
				if($divider['align'] === 'left'){
					$css .= 'margin-right:auto;';
				}
				if($divider['align'] === 'right'){
					$css .= 'margin-left:auto;';
				}
			}
			
			// Concatenación de transformaciones para evitar la sobrescritura
			$transform = '';
			if ($k === 'bottom') {
				$transform .= 'rotate(180deg)';
				if (!empty($divider['direction'])) {
					$transform .= ' scaleX(-1)';
				}
			}
			if ($k === 'top') {
				if (!empty($divider['direction'])) {
					$transform .= 'scaleX(-1)';
				}
			}
			// Agregar el estilo transform después de concatenar todas las transformaciones
			if (!empty($transform)) {
				$css .= 'transform:' . $transform . ';';
			}
		}
	

		return $css;
	}
	
	public function getBackgroundGradient($value){

		$css = "";
		if(!empty($value['gradient']['type'])){
			if($value['gradient']['type'] === 'horizontal'){
				$css .= 'background:' . 'linear-gradient(to right,  '.(($value['gradient']['start']['color'] ?? 'transparent') ?? 'transparent').' '.$value['gradient']['start']['location'].'%,'.$value['gradient']['end']['color'].' '.$value['gradient']['end']['location'].'%);';
			}
			if($value['gradient']['type'] === 'vertical'){
				$css .= 'background:' . 'linear-gradient(to bottom,  '.($value['gradient']['start']['color'] ?? 'transparent').' '.$value['gradient']['start']['location'].'%,'.$value['gradient']['end']['color'].' '.$value['gradient']['end']['location'].'%);';
			}
			if($value['gradient']['type'] === 'diagonal'){

				$css .= 'background:' . 'linear-gradient(135deg,  '.($value['gradient']['start']['color'] ?? 'transparent').' '.$value['gradient']['start']['location'].'%,'.$value['gradient']['end']['color'].' '.$value['gradient']['end']['location'].'%);';

			}
			if($value['gradient']['type'] === 'diagonal-bottom'){
				$css .= 'background:' . 'linear-gradient(45deg,  '.($value['gradient']['start']['color'] ?? 'transparent').' '.$value['gradient']['start']['location'].'%,'.$value['gradient']['end']['color'].' '.$value['gradient']['end']['location'].'%);';
			}
			if($value['gradient']['type'] === 'radial'){
				$css .= 'background:' . 'radial-gradient(ellipse at center,  '.($value['gradient']['start']['color'] ?? 'transparent').' '.$value['gradient']['start']['location'].'%,'.$value['gradient']['end']['color'].' '.$value['gradient']['end']['location'].'%);';
			}
		}

		return $css;
	}

	public function getBackgroundOverlay($value){

		$css = "";

		if($value['overlay']['type'] === 'solid'){
			$css .= 'background-color:' . $value['overlay']['start']['color'] .';' ?? 'transparent;';
		}
		if($value['overlay']['type'] === 'horizontal'){
			$css .= 'background:' . 'linear-gradient(to right,  '.(($value['overlay']['start']['color'] ?? 'transparent') ?? 'transparent').' '.$value['overlay']['start']['location'].'%,'.$value['overlay']['end']['color'].' '.$value['overlay']['end']['location'].'%);';
		}
		if($value['overlay']['type'] === 'vertical'){
			$css .= 'background:' . 'linear-gradient(to bottom,  '.($value['overlay']['start']['color'] ?? 'transparent').' '.$value['overlay']['start']['location'].'%,'.$value['overlay']['end']['color'].' '.$value['overlay']['end']['location'].'%);';
		}
		if($value['overlay']['type'] === 'diagonal'){

			$css .= 'background:' . 'linear-gradient(135deg,  '.($value['overlay']['start']['color'] ?? 'transparent').' '.$value['overlay']['start']['location'].'%,'.$value['overlay']['end']['color'].' '.$value['overlay']['end']['location'].'%);';

		}
		if($value['overlay']['type'] === 'diagonal-bottom'){
			$css .= 'background:' . 'linear-gradient(45deg,  '.($value['overlay']['start']['color'] ?? 'transparent').' '.$value['overlay']['start']['location'].'%,'.$value['overlay']['end']['color'].' '.$value['overlay']['end']['location'].'%);';
		}
		if($value['overlay']['type'] === 'radial'){
			$css .= 'background:' . 'radial-gradient(ellipse at center,  '.($value['overlay']['start']['color'] ?? 'transparent').' '.$value['overlay']['start']['location'].'%,'.$value['overlay']['end']['color'].' '.$value['overlay']['end']['location'].'%);';
		}

		$css .= 'content:""; position:absolute; top:0; left:0; width:100%; height:100%;';
		return $css;
	}

	// Generate stiles from generateCSS first
	public function generateCSS($state, $state_value, $items, $wid) {
		$css = '';

		$selectors = [];
		$config = \Drupal::config('noahs_page_builder.settings');
	
		foreach ($state_value as $item_id => $element) {

	
			if($items[$item_id]['type'] === 'noahs_multiple_elements'){

				if (!empty($element)) {
					$index = 1;
					foreach($element as $k => $style_css){


						foreach($style_css as $sk => $_css){
						

							// Reiniciar el selector en cada iteración
							$selector = '';
							$numero = null;
							if (preg_match('/_(\d+)$/', $sk, $matches)) {
								$numero = $matches[1];
							}
			
							// Generar el selector según el valor del 'style_selector'
							$selector .= ($items[$item_id]['fields'][$k]['style_selector'] === 'widget') ? 
								'#widget-id-' . $wid : 
								'#widget-id-' . $wid .' ' . $items[$item_id]['fields'][$k]['style_selector'];
			
							// Agregar el estado 'hover' si corresponde
							$selector .= ($state != 'hover') ? '' : ':hover';
	
							// Reemplazar '[index]' por el número correcto
							$selector = str_replace('[index]', $numero, $selector);

							// Reiniciar declaración CSS
							$declaration = '';

							// Condiciones para agregar declaraciones CSS según el tipo
							if (in_array($items[$item_id]['fields'][$k]['type'], ['noahs_font', 'noahs_margin', 'noahs_padding', 'noahs_radius', 'noahs_border'])) {

								foreach($element[$k] as $element_index => $item_value){
					
									$property = str_replace('_', '-', $element_index);
									if (!empty($value)) {
										$declaration .= $property . ':' . $item_value . ';';
									}
								}
							}
			
							if (in_array($items[$item_id]['fields'][$k]['type'], ['select', 'noahs_color', 'checkbox', 'noahs_width'])) {
								$declaration .= $items[$item_id]['fields'][$k]['style_css'] . ':' . $_css . ';';
							}
			
							if ($items[$item_id]['fields'][$k]['type'] === 'text') {
								$declaration .= $items[$item_id]['fields'][$k]['style_css']['text'] . ':' . $_css . ';';
							}
			
							if ($items[$item_id]['fields'][$k]['type'] === 'noahs_background_image') {
				
								// foreach($element[$k] as $element_index => $item_value){

									$background = $this->getBackgroundImage($_css);
									if (!empty($background)) {
										$declaration .= $background;
									}
								// }
							
							}
			// dump($declaration);
							if ($items[$item_id]['fields'][$k]['type'] === 'noahs_background_gradient') {
								$background = $this->getBackgroundGradient($element[$k]);
								if (!empty($background)) {
									$declaration .= $background;
								}
							}
			
							if ($items[$item_id]['fields'][$k]['type'] === 'noahs_background_overlay') {

								$background = $this->getBackgroundOverlay($_css);
								if (!empty($background)) {
									$declaration .= $background;
								}
							}
			
							// Verificar si hay una declaración CSS para agregarla al selector
							if (!empty($declaration)) {
								$selectors[$selector][] = $declaration;
							}
						}
			
						$index++;
					}
				}
			}
			

	
		

			if (!empty($items[$item_id]['style_type']) && $items[$item_id]['style_type'] === 'style') {
				
				$declaration = '';
				$selector = '';
				$resultado = '';
			
				if (!empty($element) && !empty($items[$item_id]['style_selector'])) {

					if( strpos($items[$item_id]['style_selector'], ',') !== false ) {

						$parts = explode(',', $items[$item_id]['style_selector']);
						foreach ($parts as $elemento) {
							$resultado .= '#widget-id-' . $wid . ' ' . $elemento . ', ';
						}
						$selector = rtrim($resultado, ', ');
					}else{
						$selector = $items[$item_id]['style_selector'];
						if (strpos($selector, '[widget]') !== false) {
							$selector = str_replace('[widget]', '#widget-id-' . $wid, $selector);
						} else{
							$selector = ($items[$item_id]['style_selector'] === 'widget')
							? '#widget-id-' . $wid 
							: '#widget-id-' . $wid .' '.$items[$item_id]['style_selector'];
						}
			
					}
				
			
					if (strpos($selector, 'before') !== false) {
						$selector = preg_replace('/\s*:\s*before/', ':before', $selector);
					} 
					if (strpos($selector, 'after') !== false) {
						$selector = preg_replace('/\s*:\s*before/', ':after', $selector);
					} 

					if ($state === 'hover') {

						$selector_parts = explode(',', $selector); 
						$hover_selector_parts = array_map(function($part) {

							return $part . ':hover';
						}, $selector_parts);

						$selector = implode(', ', $hover_selector_parts);
					}
					if (!empty($items[$item_id]['style_selector_hover'])) {
						$selector = '#widget-id-' . $wid .' '. $items[$item_id]['style_selector_hover'] . ',' . $selector;
					}

					if ($items[$item_id]['type'] === 'noahs_font' ||
						$items[$item_id]['type'] === 'noahs_margin' ||
						$items[$item_id]['type'] === 'noahs_padding' ||
						$items[$item_id]['type'] === 'noahs_radius' ||
						$items[$item_id]['type'] === 'noahs_border') {
							
						foreach ($element as $property => $value) {
							$property = str_replace('_', '-', $property);
							if (!empty($value)) {
								$declaration .= $property . ':' . $value . ';';
							}
						}
					}
					if ($items[$item_id]['type'] === 'noahs_shadows') {

						$shadow_x = isset($element['shadow_x']) && $element['shadow_x'] !== '' ? $element['shadow_x'] : '0';
						$shadow_y = isset($element['shadow_y']) && $element['shadow_y'] !== '' ? $element['shadow_y'] : '0';
						$blur = isset($element['blur']) && $element['blur'] !== '' ? $element['blur'] : '0';
						$spread = isset($element['spread']) && $element['spread'] !== '' ? $element['spread'] : '0';
						$color = isset($element['color']) && $element['color'] !== '' ? $element['color'] : 'black';
						$box_shadow = "{$shadow_x} {$shadow_y} {$blur} {$spread} {$color}";

						$declaration .= "box-shadow: {$box_shadow};";
				
					}

					if ($items[$item_id]['type'] === 'noahs_coordinates'){

						foreach ($items[$item_id]['fields'] as $key => $value) {
							$property = str_replace('_', '-', $key);

							if (!empty($value) && !empty($element[$key])) {
								$declaration .= $property . ':' . $element[$key] . ';';
							}
						}
					}
					
					if ($items[$item_id]['type'] === 'select' ||
						$items[$item_id]['type'] === 'noahs_color' ||
						$items[$item_id]['type'] === 'number' ||
						$items[$item_id]['type'] === 'checkbox' ||
						$items[$item_id]['type'] === 'noahs_width') {

						$style_suffix = !empty($items[$item_id]['style_suffix']) ? $items[$item_id]['style_suffix'] : null;

						$declaration .= $items[$item_id]['style_css'] . ':' . $element . $style_suffix .';';
					}
					if ($items[$item_id]['type'] === 'text') {
						$style_suffix = !empty($items[$item_id]['style_suffix']) ? $items[$item_id]['style_suffix'] : null;
						$declaration .= $items[$item_id]['style_css'] . ':' . $element['text'] . $style_suffix .';';
					}

					if ($items[$item_id]['type'] === 'noahs_image_mask' && !empty($element['mask'])) {

						$declaration .= 'mask: url('.$element['mask'].') no-repeat center;';
						$declaration .= '-webkit-mask: url('.$element['mask'].') no-repeat center;';
						$declaration .= 'mask-size: '.($element['mask_size'] ?? '100%'). ';';
						$declaration .= '-webkit-mask-size: '.($element['mask_size'] ?? '100%'). ';';
					}
	
					if ($items[$item_id]['type'] === 'noahs_transform') {

						$transforms = [];

						if (!empty($element['scale'])) {
							$transforms[] = 'scale(' . $element['scale'] . ')';
						}
						
						if (!empty($element['rotate_x'])) {
							$transforms[] = 'rotateX(' . $element['rotate_x'] . 'deg)';
						}
						if (!empty($element['rotate_y'])) {
							$transforms[] = 'rotateY(' . $element['rotate_y'] . 'deg)';
						}
						
						if (!empty($element['rotate_z'])) {
							$transforms[] = 'rotateZ(' . $element['rotate_z'] . 'deg)';
						}
						
						if (!empty($element['translate_x'])) {
							$transforms[] = 'translateX(' . $element['translate_x'] . ')';
						}
						
						if (!empty($element['translate_y'])) {
							$transforms[] = 'translateY(' . $element['translate_y'] . ')';
						}
						
						if (!empty($element['skew_x'])) {
							$transforms[] = 'skewX(' . $element['skew_x'] . 'deg)';
						}
						
						if (!empty($element['skew_y'])) {
							$transforms[] = 'skewY(' . $element['skew_y'] . 'deg)';
						}
						if (!empty($element['skew_z'])) {
							$transforms[] = 'skewZ(' . $element['skew_z'] . 'deg)';
						}
						
						$declaration .= 'transform: ' . implode(' ', $transforms) . ';';


					}
	
					if ($items[$item_id]['type'] === 'noahs_background_image') {

						$background = $this->getBackgroundImage($element);
						if (!empty($background)) {
							$declaration .= $background;
						}
				
					}
	
	
					if ($items[$item_id]['type'] === 'noahs_background_gradient') {
					
						$background = $this->getBackgroundGradient($element);
						if (!empty($background)) {
							$declaration .= $background;
						}
					}
				
					if ($items[$item_id]['type'] === 'noahs_background_overlay') {
				
						$background = $this->getBackgroundOverlay($element);
						if (!empty($background)) {
							$declaration .= $background;
						}
					}

			
				}
				if (!empty($declaration)) {

					$selectors[$selector][] = $declaration;
				}
			}
			if ($items[$item_id]['type'] === 'noahs_divider') {

					
				foreach ($element as $key => $value) {
	
					$selector = '#widget-id-' . $wid .' .noahs-divider-svg.'.$key.' svg';
	
	
					$declaration = $this->dividerStyles($key, $value);
					$css .= $selector . '{' .$declaration . '}';

				}
	
		
			}
			
			if($items[$item_id]['type'] === 'noahs_custom_css'){
				$css .= str_replace('selector', '#widget-id-' . $wid, $element);
			}
		}

		// Generate CSS from grouped selectors and declarations
		foreach ($selectors as $selector => $declarations) {
			$css .= $selector . '{' . implode('', $declarations) . '}';
		}

		return $css;
	}
	
	

}